/******************************************************************************
*  Information on one RPG class type (fighter, thief, wizard, etc.).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-22
*  @version  1.1
******************************************************************************/

public class ClassInfo {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Name of this class. */
	String name;

	/** Abbreviation for this class. */
	String abbreviation;

	/** Attack bonus numerator. */
	int atkBonusNumer;

	/** Attack bonus denominator. */
	int atkBonusDenom;
	
	/** Hit dice type (sides). */
	int hitDiceType;
	
	/** Hit dice maximum. */
	int hitDiceMax;
	
	/** Hit points added after max dice. */
	int advancedHpInc;

	/** XP increment needed for advanced levels. */
	int advancedXpInc;
	
	/** Array of low-level XP requirements. */
	int[] xpReqs;
	
	/** Array of level titles. */
	String[] titles;


	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (from InfoTables).
	*  @param idx Which row in the master table to read.
	*  @param master The master class information table.
	*  @param xpTable Table with level titles and XP for this class.
	*/
	public ClassInfo (int idx, InfoTable master, InfoTable xpTable) {
	
		// Primitive fields
		name = master.getCell(idx, 0);
		abbreviation = master.getCell(idx, 1);
		String atkBonusStr = master.getCell(idx, 2);
		String[] atkBonusPart = atkBonusStr.split(" per ");
		atkBonusNumer = Integer.parseInt(atkBonusPart[0]);
		atkBonusDenom = Integer.parseInt(atkBonusPart[1]);
		hitDiceType = master.getCellAsInt(idx, 3);
		hitDiceMax = master.getCellAsInt(idx, 4);
		advancedHpInc = master.getCellAsInt(idx, 5);
		
		// Array of titles & XP
		int arraySize = xpTable.numRows();
		titles = new String[arraySize];
		xpReqs = new int[arraySize];
		for (int i = 0 ; i < arraySize; i++) {
			titles[i] = xpTable.getCell(i, 1);		
			xpReqs[i] = xpTable.getCellAsInt(i, 2);		
		}
		
		// Compute advancedXpInc from last two levels
		advancedXpInc = xpReqs[arraySize-1] - xpReqs[arraySize-2];
	}
	
	/**
	*  Constructor (test dummy).
	*/
	public ClassInfo () {

		// Primitive fields
		name = "Fighter";
		abbreviation = "Ftr";
		atkBonusNumer = 1;
		atkBonusDenom = 1;
		hitDiceType = 8;
		hitDiceMax = 9;
		advancedHpInc = 3;

		// Array of titles & XP
		titles = new String[] {"Veteran", "Warrior", "Swordsman", "Hero"};
		xpReqs = new int[] {0, 2000, 4000, 8000};
		advancedXpInc = 4000;
	}				

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Get the class name.
	*/
	public String getName () { return name; }

	/**
	*  Get the class abbreviation.
	*/
	public String getAbbreviation () { return abbreviation; }

	/**
	*  Compute the attack bonus.
	*/
	public int getAttackBonus (int level) {
		return level * atkBonusNumer / atkBonusDenom;
	}

	/**
	*  Returns added hit dice at a given level.
	*/
	public Dice getHitDiceInc (int level) {
		if (level <= hitDiceMax)
			return new Dice(1, hitDiceType, 0);
		else
			return new Dice(0, hitDiceType, advancedHpInc);
	}

	/**
	*  Returns cumulative hit dice at a given level.
	*/
	public Dice getHitDiceTotal (int level) {
		if (level < hitDiceMax)
			return new Dice(level, hitDiceType, 0);
		else
			return new Dice(hitDiceMax, hitDiceType, 
				(level - hitDiceMax) * advancedHpInc);
	}

	/**
	*  Get the title for a level.
	*/
	public String getTitle (int level) {
		int maxTable = titles.length;
		if (level < maxTable)
			return titles[level-1];
		else
			return titles[maxTable-1];
	}

	/**
	*  Compute the experience required for a level.
	*/
	public int getExperienceReq (int level) {
		int maxTable = xpReqs.length;
		if (level < maxTable)
			return xpReqs[level-1];
		else
			return xpReqs[maxTable-1]
				+ advancedXpInc * (level - maxTable);
	}

	/**
	*  Returns a string representation of this record.
	*/
	public String toString() {
		return name + " Class "
			+ "(HD d" + hitDiceType 
			+ ", Atk " + atkBonusNumer + "/" + atkBonusDenom + ")";
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		ClassInfo info = new ClassInfo();
		System.out.println(info);
		for (int i = 1; i <= 10; i++) {
			System.out.println(i + "|" + info.getTitle(i) + "|" 
				+ info.getExperienceReq(i) + "|" + info.getAttackBonus(i));
		}
		System.out.println();
		System.out.println("HD increment for level 4: " + info.getHitDiceInc(4));
		System.out.println("HD total for level 4: " + info.getHitDiceTotal(4));
		System.out.println("HD increment for level 15: " + info.getHitDiceInc(15));
		System.out.println("HD total for level 15: " + info.getHitDiceTotal(15));
	}

	/*--------------------------------------------------------------------------
	* NOTES:
	* TODO: Armor & weapons allowed, special abilities (feats, skills, spells).
	*-------------------------------------------------------------------------*/
}

