/******************************************************************************
*  Creature (hostile or benign).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

public class Creature implements Comparable<Creature> {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------
	String name;
	String type;
	Dice numberAppearing;
	int armorClass;
	int moveInches;
	Dice hitDice;
	int inLairPct;
	String treasureType;
	Attack attack;
	String special;
	Alignment alignment;
	int monsterLevel;
	int maxHitPoints;
	int hitPoints;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (primitive stats).
	*/
	public Creature (int AC, int MV, int HD) {
		this(null, "Creature", Alignment.Neutral, AC, MV,
			new Dice(HD, 6), new Attack(HD, 1));
	}

	/**
	*  Constructor (primitive stats, attack rate, damage dice num).
	*/
	public Creature (int AC, int MV, int HD, int atkRate, int damDice) {
		this(null, "Creature", Alignment.Neutral, AC, MV,
			new Dice(HD, 6), new Attack(HD, atkRate, damDice));
	}

	/**
	*  Constructor (most fields).
	*/
	public Creature (String name, String type, Alignment alignment, 
			int AC, int MV, Dice hitDice, Attack attack) {
		this.name = name;
		this.type = type;
		this.alignment = alignment;
		this.armorClass = AC;
		this.moveInches = MV;
		this.hitDice = hitDice;
		this.attack = attack;
		rollHitPoints();
	}

	/**
	*  Constructor (from InfoTable).
	*  @param row Which row in the table to read.
	*  @param table The table of text information.
	*/
	public Creature (int row, InfoTable table) {
		name = null;
		type = table.getCell(row, 0);
		numberAppearing = new Dice(table.getCell(row, 1));
		armorClass = table.getCellAsInt(row, 2);
		moveInches = table.getCellAsInt(row, 3);
		hitDice = new Dice(table.getCell(row, 4));
		inLairPct = table.getCellAsInt(row, 5);
		treasureType = table.getCell(row, 6);
		int attackRate = table.getCellAsInt(row, 7);
		Dice damageDice = new Dice(table.getCell(row, 8));
		attack = new Attack(null, hitDice.getNumber(), 
			attackRate, damageDice);
		special = table.getCell(row, 9);
		alignment = Alignment.valueOf(table.getCell(row, 10));
		monsterLevel = table.getCellAsInt(row, 11);
		rollHitPoints();
	}

	/**
	*  Constructor (copy).
	*/
	public Creature (Creature c) {
		name = c.name;
		type = c.type;
		numberAppearing = c.numberAppearing;
		armorClass = c.armorClass;
		moveInches = c.moveInches;
		hitDice = c.hitDice;
		inLairPct = c.inLairPct;
		treasureType = c.treasureType;
		attack = c.attack;
		special = c.special;
		alignment = c.alignment;
		monsterLevel = c.monsterLevel;
		maxHitPoints = c.maxHitPoints;
		hitPoints = c.hitPoints;
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	// Primary accessors
	public String getName() { return name; }
	public String getType() { return type; }
	public Dice getNumberAppearing() { return numberAppearing; }
	public int getArmorClass() { return armorClass; }
	public int getMoveInches() { return moveInches; }
	public int getHitPoints() { return hitPoints; }
	public int getMaxHitPoints() { return maxHitPoints; }
	public int getInLairPct () { return inLairPct; }
	public Dice getHitDice () { return hitDice; }
	public String getTreasureType () { return treasureType; }
	public Attack getAttack () { return attack; }
	public String getSpecial () { return special; }
	public Alignment getAlignment() { return alignment; }
	public int getMonsterLevel () { return monsterLevel; }

	// Shortcut accessors
	public int getAC() { return getArmorClass(); }
	public int getHD() { return getHitDiceNum(); }
	public int getHP() { return getHitPoints(); }
	public int getMV() { return getMoveInches(); }

	/**
	*  Compare to another creature by hit dice num.
	*/
   public int compareTo(Creature other) {
		int myHD = getHitDiceNum();
		int otherHD = other.getHitDiceNum();
		if (myHD > otherHD) 
			return 1;
		else if (myHD < otherHD)
			return -1;
		else
			return 0;
    }

	/**
	*  Return raw number of hit dice, accounting for fractions.
	*/
	public int getHitDiceNum () {
		int hd = hitDice.getNumber();
		int mul = hitDice.getMultiplier();
		if (mul >= 0)
			hd *= mul;
		else
			hd /= (-mul);
		return hd;
	}

	/**
	*  Roll hit points from hit dice.
	*/
	private void rollHitPoints() {
		maxHitPoints = hitDice.roll();
		hitPoints = maxHitPoints;
	}

	/**
	*  Take damage (minimum 0 hp).
	*/
	public void takeDamage (int damage) {
		hitPoints -= damage;
		if (hitPoints < 0)
			hitPoints = 0;
	}

	/**
	*  Check if the creature is dead.
	*/
	public boolean isDead() {
		return hitPoints <= 0;	
	
	}

	/**
	*  Set hit points to maxHitPoints.
	*/
	public void setHitPointsToMax () {
		hitPoints = getMaxHitPoints();
	}

	/**
	*  Add XP to a creature: do nothing (subclass overrides this).
	*/
	public void addXP (int XP) {
	}

	/**
	*  Spawn a new creature of the same type, but new hit points.
	*/
	public Creature spawn () {
		Creature c = new Creature(this);
		c.rollHitPoints();
		return c;
	}

	/**
	*  Return a string representation of this creature.
	*/
	public String toString() {
		return (name != null ? name + ", " : "")
		  + type + " (AC " + armorClass + ", MV " + moveInches 
		  + ", HD " + hitDice.getNumber() + ", hp " + hitPoints 
		  + ", Atk " + attack.getRate() + ", Dam " + attack.getDamage()
		  + ")";
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		Creature c = new Creature(5, 9, 4);
		System.out.println(c);
	}

	/*--------------------------------------------------------------------------
	* TODO (at some point):
	* Multiple attack forms (in ArrayList, possibly persona can register/swap)
	* moveFly, moveSwim (etc.?)
	*-------------------------------------------------------------------------*/
}

