/******************************************************************************
*  Information table (as in an RPG).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-07-15
*  @version  1.0
******************************************************************************/

import java.io.*; 
import java.util.Scanner;
import java.util.ArrayList;


public class InfoTable {
	//-----------------------------------------------------------------
	//  Constants
	//-----------------------------------------------------------------

	/** Marker for new table in CSV file. */
	static public final String TABLE_START = "TABLE: ";

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Name of the table. */
	private String name;
	
	/** Headers for each column in the table. */
	private String[] headers;	

	/** Body of the table (row, column). */
	private String[][] cells;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**	
	*  Constructor (empty).
	*/
	InfoTable () {
		name = "Empty Table";
		headers = new String[0];
		cells = new String[0][];		
	}

	/**	
	*  Constructor (read in from open CSV file).
	*  @param name Name of the table.
	*  @param scan Scanner, pointed at header line in the file.
	*/
	InfoTable (String name, Scanner scan) throws IOException {

		// Set the name
		this.name = name;

		// Read headers
		headers = new String[0];
		if (scan.hasNextLine()) {
			headers = CSVReader.splitLine(scan.nextLine());
		}

		// Read rows (end on blank line)
		ArrayList<String> rowList = new ArrayList<String>();
		while (scan.hasNextLine()) {
			String s = scan.nextLine();
			String trim = CSVReader.trimTrailingDelimit(s);
			if (trim.length() > 0)
				rowList.add(s);
			else
				break;
		}

		// Parse out cells
		cells = new String[rowList.size()][];
		for (int i = 0; i < rowList.size(); i++) {
			cells[i] = CSVReader.splitLine(rowList.get(i));
		}
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public String name () { return name; };
	public String toString () { return name; };
	public String getHeader (int i) { return headers[i]; };
	public String getCell (int row, int col) { return cells[row][col]; };
	public int numRows () { return cells.length; };
	public int numColumns () { return headers.length; };

	/**
	*  Return a cell as an integer value.
	*  A single or double dash is read as zero (0).
	*/
	public int getCellAsInt (int row, int col) {
		String s = getCell(row, col);
		if (s.equals("-") || s.equals("--"))
			return 0;
		else
			return Integer.parseInt(s);
	};

	/**
	*  Read a file containing multiple tables.
	*  @param filename File to read.
	*  @return Array of InfoTables.
	*/
	public static InfoTable[] readTableFile (String filename) throws IOException {
		ArrayList<InfoTable> tableList = new ArrayList<InfoTable>();
		Scanner scan = new Scanner(new File(filename));
		while (scan.hasNextLine()) {
			String s = scan.nextLine();
			if (s.startsWith(TABLE_START)) {
				scan.nextLine(); 
				s = CSVReader.trimTrailingDelimit(s);
				String name = s.substring(TABLE_START.length());
				tableList.add(new InfoTable(name, scan));
			}
		}
		scan.close();
		InfoTable[] format = new InfoTable[0];
		return tableList.toArray(format);
	}

	/**
	*  Main test function.
	*/
	public static void main (String[] args) throws IOException {

		// Read & report on the table test file
		InfoTable[] tables = readTableFile("TableTest.csv");
		for (InfoTable t: tables) {
			System.out.println(t.name());
			for (int i = 0; i < t.numColumns(); i++) {
				System.out.print(t.getHeader(i) + "|");
			}
			System.out.println();
			System.out.println(t.numRows() + " x " + t.numColumns());
			System.out.println(t.getCellAsInt(0, 1));
			System.out.println();
		}
	}
}

