/******************************************************************************
*  Persona (player or non-player character).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

import java.util.ArrayList;

public class Persona extends Creature {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** The six ability scores. */
	int [] abilityScore;

	/** List of class records gained. */
	ArrayList<ClassRecord> classRecordList;

	/** Which method to use for ability bonuses (0=BX, 1=OED). */
	static final int BONUS_METHOD = 1;

	/** Whether we apply the prime-requisite XP bonus. */
	static final boolean APPLY_BONUS_XP = false;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (human, 1st level in some class).
	*/
	public Persona (String name, Alignment alignment, ClassInfo classInfo) {
		super(9, 12, 1);

		// Basics
		this.name = name;
		this.type = "Human";
		this.alignment = alignment;
		createRandomAbilityScores();
		classRecordList = new ArrayList<ClassRecord>();
		classRecordList.add(new ClassRecord(classInfo, getBonus(Ability.Con)));
		setHitPointsToMax();

		// Clunky switch on class HD (TODO: improve)
		switch (classInfo.getHitDiceInc(1).getSides()) {
			case 8: // Fighter
				moveInches = 9;
				armorClass = 2 - getBonus(Ability.Dex); // plate & shield
				attack = new Attack("Sword", 1 + getBonus(Ability.Str), 
					1,	1, 8, getBonus(Ability.Str));
				break;

			case 6: // Thief
				moveInches = 12;
				armorClass = 7 - getBonus(Ability.Dex); // leather
				attack = new Attack("Sword", 0 + getBonus(Ability.Str), 
					1,	1, 8, getBonus(Ability.Str));
				break;

			case 4: // Wizard
				moveInches = 12;
				armorClass = 9 - getBonus(Ability.Dex); // no armor
				attack = new Attack("Dagger", 0 + getBonus(Ability.Str), 
					1,	1, 4, getBonus(Ability.Str));
				break;
		}
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Creates random ability scores.
	*/
	private void createRandomAbilityScores () {
		abilityScore = new int[Ability.values().length];
		Dice dice = new Dice(3, 6);
 		for (int i = 0; i < abilityScore.length; i++) {
 			abilityScore[i] = dice.roll();
 		}
	}

	/**
	*  Gets an ability score.
	*/
	public int getScore (Ability ability) {
		return abilityScore[ability.ordinal()];	
	}

	/**
	*  Gets the ability score by index.
	*/
	public int getScore (int index) {
		return abilityScore[index];	
	}

	/**
	*  Gets an ability score bonus/modifier.
	*/
	public int getBonus (Ability ability) {
		int score = getScore(ability);
		return abilityBonus(score);
	}

	/**
	*  Gives the bonus for a given ability score.
	*/
	private static int abilityBonus (int score) {
		if (BONUS_METHOD == 1)
			return abilityBonus_OED(score);
		else
			return abilityBonus_BX(score);	
	}

	/**
	*  Gives the bonus for a given ability score (OED-style).
	*/
	private static int abilityBonus_OED (int score) {
		if (score >= 13) 
			return (score - 10)/3;
		else if (score <= 8) 
			return (score - 11)/3;
		else 
			return 0;
	}

	/**
	*  Gives the bonus for a given ability score (BX-style).
	*/
	private static int abilityBonus_BX (int score) {
		switch (score) {
			case 3: return -3;
			case 4: case 5: return -2;
			case 6: case 7: case 8: return -1;
			case 9: case 10: case 11: case 12: return 0;
			case 13: case 14: case 15: return +1;
			case 16: case 17: return +2;
			case 18: return +3;

			// Note: Mentzer's Immortals rules has a 
			// lunatic system for ability scores 1-100.
			default: return Integer.MAX_VALUE;
		}
	}

	/**
	*  Returns max level as hit dice number (max over all classes).
	*  TODO: Record in hitDice field at appropriate time. (?)
	*  ## This is probably the clunkiest bit here. Also want level return? ##
	*/
	public int getHitDiceNum () {
		int maxLevel = 0;
		for (ClassRecord record: classRecordList) {
			if (record.getLevel() > maxLevel)
				maxLevel = record.getLevel();
		}
		return maxLevel;
	}

	/**
	*  Returns max hit points (max over all classes).
	*  TODO: Record in maxHitPoints field at appropriate time.
	*/
	public int getMaxHitPoints () {
		int maxHP = 0;
		for (ClassRecord record: classRecordList) {
			if (record.getHitPoints() > maxHP)
				maxHP = record.getHitPoints();		
		}
		return maxHP;
	}

	/**
	*  Returns base attack bonus (max over all classes).
	*/
	public int baseAttackBonus () {
		int maxBAB = 0;
		for (ClassRecord record: classRecordList) {
			if (record.attackBonus() > maxBAB)
				maxBAB = record.attackBonus();
		}
		return maxBAB;
	}

	/**
	*  Returns total XP (sum over all classes).
	*/
	public int totalXP () {
		int sum = 0;
		for (ClassRecord record: classRecordList) {
			sum += record.getXP();
		}
		return sum;
	}

	/**
	*  Add XP to this persona.
	*  TODO: Currently adds to & updates only the first class.
	*/
	public void addXP (int XP) {
		if (APPLY_BONUS_XP) {
			XP = XP + XP * bonusXPpct()/100;
		}
		ClassRecord record = classRecordList.get(0);
		record.addXP(XP, getBonus(Ability.Con));
		Attack newAttack = new Attack(attack.getName(),
			record.attackBonus() + getBonus(Ability.Str),
			1, attack.getDamage());
		attack = newAttack;
	}

	/**
	*  Returns bonus XP percent for prime ability score.
	*/
	private int bonusXPpct () {
		int primeReq = primeRequisite();
		if (primeReq >= 15) return 10;
		else if (primeReq >= 13) return 5;
		else if (primeReq >=9) return 0;
		else if (primeReq >= 7) return -10;
		else return -20;
	}

	/**
	*  Returns the effective prime requisite score.
	*  TODO: Currently assumes fighter class, generalize for given class.
	*    Possibly shove this down into ClassInfo or ClassRecord.
	*/
	private int primeRequisite () {
		// Raw prime requisite score
		return getScore(Ability.Str);

// 		// Vol-1 modifiers to prime requisite
// 		return getScore(Ability.Str) 
// 			+ getScore(Ability.Int)/2 
// 			+ getScore(Ability.Wis)/3;
	}

	/**
	*  Returns a string representation of this persona.
	*/
	public String toString() {
		return (name == null ? "" : name + ", ")
			+ (type == null ? "" : type + " ")
			+ classString() + " "
			+ "(AC " + getAC() + ", MV " + getMV() + ", HD " + getHD() 
			+ ", hp " + getHP() + ", Atk " + attack + "; " 
			+ abilityString() + "; XP " + totalXP() + ")";
	}	

	/**
	*  String representation of all class & levels.
	*/
	public String classString() {
		String s = "";
		for (ClassRecord record: classRecordList) {
			s += record.getInfo().getAbbreviation() + record.getLevel();
			if (record != classRecordList.get(classRecordList.size()-1))
				s += "/";
		}
		return s;	
	}

	/**
	*  String representation of all ability scores.
	*/
	public String abilityString() {
		String s;
		s = "Str " + abilityScore[0] + ", Int " + abilityScore[1]
			+ ", Wis " + abilityScore[2] + ", Dex " + abilityScore[3]
			+ ", Con " + abilityScore[4] + ", Cha " + abilityScore[5];
		return s;
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		Persona p = new Persona(null, Alignment.Neutral, ClassList.getInfo("Fighter"));
		System.out.println(p);
	}
	
	/*--------------------------------------------------------------------------
	* TODO:
	* Properly equipped armor, shield, weapon.
	* Later: Languages, special abilities, equipment list, encumbrance.
	* Note: Not called "character" to not conflict with java.lang.Character
	* Using Ability enum array saves maintaining 12 individual methods.
	*-------------------------------------------------------------------------*/
}

